// McfSerializer.cpp: implementation of the McfSerializer class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#include "McfSerializer.h"
#include <comutil.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

McfSerializer::McfSerializer(): m_pcaArchive(0),
                                m_cmfDataFile(),
                                         m_pbyBuffer( 0 ),
                                         m_bValid ( false )
{

	m_pcaArchive = new CArchive( &m_cmfDataFile,CArchive::store );
            if( 0 == m_pcaArchive )
            {
                return;
            }
            m_bValid = true;
	 
           

}

McfSerializer::~McfSerializer()
{
	m_cmfDataFile.Close();
            if( 0 != m_pcaArchive )
            {
                delete m_pcaArchive;
                m_pcaArchive = 0;
            }

            if( 0 != m_pbyBuffer )
            {
                delete[] m_pbyBuffer;
                m_pbyBuffer = 0;
            }

}

McfSerializer::McfSerializer(const VARIANT &vtData_i): m_pcaArchive(0),	
														m_cmfDataFile(),  m_pbyBuffer( 0 ),m_bValid( false )
{
	 try
            {
                // attach the input variant to a SafeArray
                COleSafeArray cosaData( vtData_i );
                DWORD dwBufferSize = 0;
                // get the byte stream from COleSafeArray
                BYTE* pbyVarBuffer = 0;
                cosaData.AccessData( reinterpret_cast<void**>( &pbyVarBuffer ));
                dwBufferSize = cosaData.GetOneDimSize();
                m_pbyBuffer = new BYTE[dwBufferSize];
                if( 0 == m_pbyBuffer )
                {
                    return;
                }
                // get the buffer
                ::CopyMemory( m_pbyBuffer, pbyVarBuffer, dwBufferSize );
                cosaData.UnaccessData();
                // attach the byte stream to memory file
                m_cmfDataFile.Attach( m_pbyBuffer, dwBufferSize );
                m_pcaArchive = new CArchive( &m_cmfDataFile,CArchive::load );
                if( 0 ==0)
                {
                    return;
                }
                m_bValid = true;
            }
            catch( ... )
            {
                
            }

}


 bool McfSerializer::IsValid()
   {
            return m_bValid;
   }


bool McfSerializer::GetVariant( VARIANT& vtData_o )
{
            try
            {
                // This line is important, it flushes the contents from CArhive object to memory file.
                m_pcaArchive->Close();
                int nBufLen = m_cmfDataFile.GetLength();
                m_pbyBuffer = m_cmfDataFile.Detach();
                // wrap the byte array in a COleSafeArry.
                COleSafeArray cosaByteArray;
                cosaByteArray.CreateOneDim( VT_UI1,nBufLen,m_pbyBuffer );
                vtData_o = cosaByteArray.Detach();
            }
            catch( ... )
            {
                // Error handling...
                return false;
            }
            return true;
 }

 McfSerializer::operator << ( char cVal_i )
 {
         (*m_pcaArchive)<<cVal_i;
 }
 
 McfSerializer::operator << ( unsigned char ucVal_i )
 {
            (*m_pcaArchive)<<ucVal_i;
 }


  McfSerializer::operator << ( short sVal_i )
        {
            (*m_pcaArchive)<<sVal_i;
        }

  /** 
         * 
         * Push a unsigned short value.
         * 
         * @param       usVal_i - Input unsigned short value
         * @return      Nil
         * @exception   Nil
         * @see         Nil
         * @since       1.0
         */
        McfSerializer::operator << ( unsigned short usVal_i )
        {
            (*m_pcaArchive)<<usVal_i;
        }


        /** 
         * 
         * Push a int value.
         * 
         * @param       nVal_i - Input int value
         * @return      Nil
         * @exception   Nil
         * @see         Nil
         * @since       1.0
         */
        McfSerializer::operator << ( int nVal_i )
        {
            (*m_pcaArchive)<<nVal_i;
        }


        /** 
         * 
         * Push a unsigned int value.
         * 
         * @param       uVal_i - Input unsigned int value
         * @return      Nil
         * @exception   Nil
         * @see         Nil
         * @since       1.0
         */
        McfSerializer::operator << ( unsigned int uVal_i )
        {
            (*m_pcaArchive)<<uVal_i;
        }


        /** 
         * 
         * Push a long value.
         * 
         * @param       lVal_i - Input long value
         * @return      Nil
         * @exception   Nil
         * @see         Nil
         * @since       1.0
         */
        McfSerializer::operator << ( long lVal_i )
        {
            (*m_pcaArchive)<<lVal_i;
        }


        /** 
         * 
         * Push a unsigned long value.
         * 
         * @param       ulVal_i - Input unsigned long value
         * @return      Nil
         * @exception   Nil
         * @see         Nil
         * @since       1.0
         */
        McfSerializer::operator << ( unsigned long ulVal_i )
        {
            (*m_pcaArchive)<<ulVal_i;
        }


        /** 
         * 
         * Push a float value.
         * 
         * @param       fVal_i - Input float value
         * @return      Nil
         * @exception   Nil
         * @see         Nil
         * @since       1.0
         */
        McfSerializer::operator << ( float fVal_i )
        {
            (*m_pcaArchive)<<fVal_i;
        }


     
        McfSerializer::operator << ( double dVal_i )
        {
            (*m_pcaArchive)<<dVal_i;
        }

       
        McfSerializer::operator << ( bool bVal_i )
        {
            (*m_pcaArchive) << static_cast<char>( bVal_i );
        }


        McfSerializer::operator << ( const CString& csVal_i )
        {
            (*m_pcaArchive)<<csVal_i;
        }


        McfSerializer::operator << ( CObject& Object_i )
        {
            Object_i.Serialize( *m_pcaArchive );
        }


      
        McfSerializer::operator >> ( char& cVal_o )
        {
            (*m_pcaArchive)>>cVal_o;
        }


       
        McfSerializer::operator >> ( unsigned char& ucVal_o )
        {
            (*m_pcaArchive)>>ucVal_o;
        }


        McfSerializer::operator >> ( short& sVal_o )
        {
            (*m_pcaArchive)>>sVal_o;
        }


     
        McfSerializer::operator >> ( unsigned short& usVal_o )
        {
            (*m_pcaArchive)>>usVal_o;
        }


        
        McfSerializer::operator >> ( int& nVal_o )
        {
            (*m_pcaArchive)>>nVal_o;
        }


    
        McfSerializer::operator >> ( unsigned int& uVal_o )
        {
            (*m_pcaArchive)>>uVal_o;
        }


        McfSerializer::operator >> ( long& lVal_o )
        {
            (*m_pcaArchive)>>lVal_o;
        }


       
        McfSerializer::operator >> ( unsigned long& ulVal_o )
        {
            (*m_pcaArchive)>>ulVal_o;
        }


       
        McfSerializer::operator >> ( float& fVal_o )
        {
            (*m_pcaArchive)>>fVal_o;
        }


        McfSerializer::operator >> ( double& dVal_o )
        {
            (*m_pcaArchive)>>dVal_o;
        }


        McfSerializer::operator >> ( bool& bVal_o )
        {
            (*m_pcaArchive) >> *( reinterpret_cast<char*>( &bVal_o) );
        }


     
        McfSerializer::operator >> ( CString& csVal_o )
        {
            (*m_pcaArchive)>>csVal_o;
        }


       
        McfSerializer::operator >> ( CObject& Object_o )
        {
            Object_o.Serialize( *m_pcaArchive );
        }


